/*
 * piQa Services Collection
 * Created on Jul 14, 2004
 * Copyright � 2004 - 2008 Ronald D. Grindle
 */
 
package RTObj;

import RTtools.UserSrvcTool;

import com.qanavigation.Basics.Rslt;
import com.qanavigation.Basics.RsltString;
import com.qanavigation.Basics.RsltTable;
import com.qanavigation.Basics.SFormatter;
import com.qanavigation.DBacc.DBacc2;
import com.qanavigation.QNservices.QNDefs;
import com.qanavigation.QNservices.QNRepBld;
import com.qanavigation.QNservices.SrvltReqTools;
import com.qanavigation.XMLBasics.RsltXMLDoc;
import com.qanavigation.XMLBasics.RsltXMLNode;
import com.qanavigation.XMLBasics.RsltXMLNodeLst;
import com.qanavigation.XMLDB.XMLDB;
import com.qanavigation.XMLDB.XMLDBNode;

/** Builds an error report.</p>  
 * <p>ProbRepBld isA Rslt.
 * @author RDG
 */
public class TRRepBld extends QNRepBld {
	
	/**
	 * 
	 * @param p_SRT
	 * @param p_FilePath
	 */
	public TRRepBld( DBacc2 p_DBacc, SrvltReqTools p_SRT, String p_FilePath ) {

		super( p_DBacc, p_SRT, p_FilePath, QNDefs.c_NodeTechReport );
				
		if( notOK() ) {
			
			addRsltAddFailInfo( "TRRepBld", "TRRepBld", "1", "" );
		}
	}
	
	/**
	 * 
	 * @param p_xDB
	 * @param p_SRT
	 * @param p_FilePath
	 */
	public TRRepBld( XMLDB p_xDB, SrvltReqTools p_SRT, String p_FilePath ) {

		super( p_xDB, p_SRT, p_FilePath, QNDefs.c_NodeTechReport );
				
		if( notOK() ) {
			
			addRsltAddFailInfo( "TRRepBld", "TRRepBld", "2", "" );
		}
	}
	
	/**
	 * Saves to DB and appends UID to TC Execution, if coming from execution.
	 * Appends files to report, if necessary.
	 * @param p_Author
	 * @param p_IsDraft
	 * @param p_wTicket
	 * @return
	 */
	public RsltString save2DB( String p_Author, boolean p_IsDraft, boolean p_wTicket ) {
		
		RsltString UID;
		
		UID = super.save2DB( p_Author, p_IsDraft, p_wTicket );
		
		if( UID.isOK() & m_bIsNewReady ) {
			
			Rslt res = appUIDtoTCEx( UID, m_bTolerant );
			
			UID.checkRslt( res );			
		}
		
		if( UID.notOK() ) {
			
			UID.addRsltAddFailInfo( "TRRepBld", "save2DB", "", "" );
		}
		
		return UID;
	}
	
	//-------------------------------------
	private Rslt appUIDtoTCEx( RsltString p_UID, boolean p_BTolerant ) {
		
		Rslt res;
		
		RsltXMLNode aNode;
		
		res = new Rslt( true );
		
		if( m_SRT.hasVal( "TCExUID" ) ) {
			
			aNode = new RsltXMLNode( "RefProbRep" );
			
			aNode.addAttribute( "RefUID", p_UID.getString() );
			
			if( res.checkRslt( aNode ) ) {
				
				appAttr( aNode, "Title", p_BTolerant );
				
				appNodeByUIDfromReq( "TCExUID", aNode );
				
				res.checkRslt( this );
			}
		
			if( res.notOK() ) {
				
				res.addRsltAddInfo( "'ProbRepBld.appUIDtoTCEx(..)' failed." );
				this.setRsltStat( res );
			}
		}

		return res;
	}
		
	/**
	 * Service to build a PR from the posted data.
	 * "Assignd" is evaluated by the service.
	 * "TCExTitle", "Appl" and "Fctn" is retrieved from the TC, if p_RetTCData is switched.
	 * "SysConfig" is appended as the SysConfigRef node, if provided. "SysConfig" is used for the title. 
	 * @param p_IsDraft
	 */
	public Rslt bldProbRep( boolean p_IsDraft, boolean p_IsSub ) {
		
		boolean bTolerant = false;
		boolean notNew = true;
		
		
		String tcExUID;
		String appl;
		String softRel;		
		String val;
		
		Rslt res = new Rslt( true );
		Rslt aRes;
		RsltString rVal;
		
		RsltXMLNode aNode;
		
// res.addRsltAddInfo( "bldProbRep0" );		
		
		try {
			
			if( p_IsDraft ) {
				
				bTolerant = true;			
			}
			
// res.addRsltAddInfo( "bTolerant: " + bTolerant);				
			
			m_xDoc = new RsltXMLDoc( "", QNDefs.c_NodeTechReport, QNDefs.c_TechRepDTD, null );
			
// res.addRsltAddInfo( "bldProbRep1" );

			if(	res.checkRslt( m_xDoc ) ) {
				
				RsltXMLNode tNode = m_xDoc.getTopNode();
			
// res.addRsltAddInfo( "bldProbRep2" );				
				res.checkRslt( tNode );
				
				//res.checkRslt( m_SRT );
				
				appAttr( tNode, "Author", false );
				 
				appAttr( tNode, "CRNum", true );
								
				appDateTime( tNode, "Date", "Time", "DateTime", bTolerant );
				
				appAttr( tNode, "ExSys", true );
				
				appAttr( tNode, "ExtID", true );
				
				appAttr( tNode, "IsCR", true );
				
				appAttr( tNode, "Prio", true );
				
				appAttr( tNode, "ProbClass", bTolerant );
				
				appAttr( tNode, "ProbCode", bTolerant );
				
				appAttr( tNode, "ProbStat", bTolerant );
				
				appAttr( tNode, "PSource", true );
				
				rVal = m_SRT.getVal( "SoftRel" );
				softRel = SFormatter.encodeHTML( rVal.getString() );
				tNode.addAttribute( "SoftRel", softRel );
				
				appAttr( tNode, "Ticket", true );
				
//rVal = m_SRT.getVal( "Title" );
//System.out.println( "TR Title '" + rVal.getString() + "'" );
				
				appAttr( tNode, "Title", bTolerant );
				
				rVal = m_SRT.getVal( "UID" );
// res.addRsltAddInfo( "bldProbRep3" );				
				if( rVal.getString().length() > 0 ) {
					
					tNode.addAttribute( "UID", rVal.getString() );
					
					//addRsltAddInfo( "UID added: " + rVal.getString() );
				}
				else {
					
					notNew = false;
				}
				
				// TC related data
					
				rVal = m_SRT.getVal( "TCExUID" );
				
				tcExUID = rVal.getString();
// res.addRsltAddInfo( "bldProbRep4" );	
				if(	tcExUID.length() > 0 ) {
				
					aNode = tNode.appNode( "RefTCEx" );
					
					appAttr( aNode,  "TCExUID", "RefUID", bTolerant );
					
					appAttrRetr( aNode, "Title", "TCExTitle", "Title", tcExUID );
					
					rVal = m_xDB.getAttrVal( tcExUID, "Appl" );
					appl = rVal.getString();
					tNode.addAttribute( "Appl", softRel );
					
					appAttrRetr( tNode, "Fctn", "Fctn", "Fctn", tcExUID );
					
					//not yet suppported: appAttrRetr( tNode, "UseCase", "UseCase", "UseCase", tcExUID );
					//not yet suppported: appAttrRetr( tNode, "UCSection", "UCSection", "UCSection", tcExUID );
				}
				else {
					
					rVal = m_SRT.getVal( "Appl" );
					appl = SFormatter.encodeHTML( rVal.getString() );
					tNode.addAttribute( "Appl", appl );	
					
					appAttr( tNode, "Fctn", bTolerant );
					
					appAttr( tNode, "UseCase", bTolerant );
					appAttr( tNode, "UCSection", bTolerant );					
				}
				
				appTxtNode( tNode, "WDYD", true );		
				appTxtNode( tNode, "Expctd", true );
				appTxtNode( tNode, "Observ", true );
				
				if( m_SRT.hasVal( "ExtTxt" ) ) {
					
					rVal = m_SRT.getVal( "ExtTxt" );
					aNode = tNode.appTxtNode( "ExtTxt", rVal.getString() );
					
					appAttr( aNode, "ExtTxtID", "ExtID", false );
					appAttr( aNode, "ExtTxtDT", "ExtDT", false );
				}
				
				if( m_SRT.hasVal( "SysConfig" ) ) {
					
					/* <!ELEMENT SysConfigRef (#PCDATA)>
					<!ATTLIST SysConfigRef
						RefUID	NMTOKEN #IMPLIED
						Title		CDATA #REQUIRED
						UID			NMTOKEN #IMPLIED
					> */
					
					aNode = tNode.appTxtNode( "SysConfigRef", "" );
					
					appAttr( aNode, "SysConfig", "Title", false );
				}
				
// res.addRsltAddInfo( "bldProbRep5" );				
				if( notNew ) {
					
					if( res.checkRslt( this ) ) {
					
						aRes = appExistFiles( tNode );
						
						if( res.checkRslt( aRes, true ) ) {
						
							res.checkRslt( appExistLogs( tNode ) );					
						}
					}
				
// res.addRsltAddInfo( "bldProbRep6" );				
					if( res.isOK() ) {
						
// res.addRsltAddInfo( "bldProbRep6.1" );						
						aRes = appExistFix( tNode );
						
						res.checkRslt( aRes );
					}
				}
				
// res.addRsltAddInfo( "bldProbRep7" );				
				if( res.isOK() ) {
				
// Branch:TCom PSys ##################
/*
 * 	<!ELEMENT PSys (EMPTY)>
		<!ATTLIST PSys
			Origin		NMTOKEN #IMPLIED		
			PSystem		NMTOKEN #REQUIRED
			Version		CDATA #REQUIRED
			UID				NMTOKEN #IMPLIED
		>
 */				
					if( m_SRT.hasVal( "PSystem" ) ) {
						
						aNode = tNode.appNode( "PSys" );
						
						appAttr( aNode, "POrigin", "Origin", true );	
						
						appAttr( aNode, "PSystem",  true );
						
						appAttr( aNode, "PSysVer", "Version", true );	
					}					
// Branch:TCom PSys End ##################
					
					if( p_IsDraft ) {
						
// res.addRsltAddInfo( "bldProbRep8" );						
						
						//tNode.addAttribute( "Assignd", "" );
						appAttr( tNode,  "Author", "Assignd", false );
					}
					else if( p_IsSub ){
						
						UserSrvcTool ust = new UserSrvcTool( m_xDB );
						
						rVal = ust.getAss4NewTR( appl, softRel );
						
// res.addRsltAddInfo( "bldProbRep9" );						
						if( res.checkRslt( rVal ) ) {
							
							tNode.addAttribute( "Assignd", rVal.getString() );	
						}
						
						// is logged outside (no RscBundle)					
					}
					else {
						
						appAttr( tNode, "Assignd", false );
					}
				}
				
// res.addRsltAddInfo( "bldProbRep10" );				
				//res.checkRslt( this );
				
				/*if( res.isOK() ) {
					
					m_IsLoaded = true;
				}*/
			}
		}
		catch( Exception excp ) {
			
			res.setRsltByExcp( excp );
		}
		
// res.addRsltAddInfo( "bldProbRep11" );		
		if( res.notOK() ) {

			res.addRsltAddFailInfo( "TRRepBld", "bldProbRep", "", "" );
		}
		else {
			
			m_XDocIsInit = true;
		}
		
		//checkRslt( res );
		
		return res;
	}	

	/**
	 * 
	 * @param p_UID
	 * @return
	 */
	public Rslt loadPRData( String p_UID ) {
		
		Rslt res = new Rslt( true );
		RsltString val;
		RsltTable attrs;
		
		RsltXMLNode aNode;
		RsltXMLNodeLst rNLst;
		
		XMLDBNode xNode;
		
		xNode = m_xDB.getXMLsubTree( p_UID );
		
		if( res.checkRslt( xNode ) ) {
			
			m_xDoc = xNode.m_RxDoc;
			
			attrs = m_xDB.getNodeAttrs( p_UID );
			
			if( res.checkRslt( attrs ) ) {
				
				m_SRT.add2ParamLst( attrs );
			}
			
			rNLst = xNode.getSubNodes( QNDefs.c_NodeRefTCEx );
			
			if( rNLst.isOK() ) {
				
				aNode = rNLst.getNode( 0 );
				
				val = aNode.getAttrVal( QNDefs.c_AttrRefUID );
				
				m_SRT.add2ParamLst( "TCExUID", val.getString() );
				
				val = aNode.getAttrVal( QNDefs.c_AttrTitle );
				
				m_SRT.add2ParamLst( "TCExTitle", val.getString() );
			}
			
			//val = m_xDB.getNodeValByParent( QNDefs.c_NodeWDYD, p_UID );
			val = xNode.getSubNodeVal( QNDefs.c_NodeWDYD );
			
//String s = val.getRsltFinalMsg() + " "	+ RsltStatVal.label[ val.getRsltStatVal() ];		
//m_SRT.add2ParamLst( QNDefs.c_NodeWDYD, s );
			
			//if( res.checkRslt( val, true ) ) {
				
				m_SRT.add2ParamLst( QNDefs.c_NodeWDYD, val.getString() );
			//}
			
			//val = m_xDB.getNodeValByParent( QNDefs.c_NodeExpctd, p_UID );
			val = xNode.getSubNodeVal( QNDefs.c_NodeExpctd );
			
			//if( res.checkRslt( val, true ) ) {
				
				m_SRT.add2ParamLst( QNDefs.c_NodeExpctd, val.getString() );
			//}
			
			//val = m_xDB.getNodeValByParent( QNDefs.c_NodeObserv, p_UID );
			val = xNode.getSubNodeVal( QNDefs.c_NodeObserv );
			
			//if( res.checkRslt( val ) ) {
				
				m_SRT.add2ParamLst( QNDefs.c_NodeObserv, val.getString() );
			//}
			
			rNLst = xNode.getSubNodes( "ExtTxt" );
			
			if( rNLst.size() > 0 ) {
				
				aNode = rNLst.getNode( 0 );
			
				val = aNode.getText();
					
				m_SRT.add2ParamLst( QNDefs.c_NodeExtTxt, val.getString() );
				
				val = aNode.getAttrVal( QNDefs.c_AttrExtID );
				
				if( val.isOK() ) {
				
					m_SRT.add2ParamLst( "ExtTxtID", val.getString() );
				}
				
				val = aNode.getAttrVal( QNDefs.c_AttrExtDT );
				
				if( val.isOK() ) {
				
					m_SRT.add2ParamLst( "ExtTxtDT", val.getString() );
				}				
			}
			
// Branch:TCom PSys ##################
			
			rNLst = xNode.getSubNodes( "PSys" );
			
			if( rNLst.isOK() ) {
				
				aNode = rNLst.getNode( 0 );
				
				val = aNode.getAttrVal( "Origin" );
				
				m_SRT.add2ParamLst( "POrigin", val.getString() );
				
				val = aNode.getAttrVal( "PSystem" );
				
				m_SRT.add2ParamLst( "PSystem", val.getString() );
				
				val = aNode.getAttrVal( "Version" );
				
				m_SRT.add2ParamLst( "PSysVer", val.getString() );				
			}
			
// Branch:TCom PSys End ##################			
			
			if( res.isOK( true ) ) {
				
//				m_IsLoaded = true;
				m_XDocIsInit = true;
			}
		}
		
		if( res.isOK() ) {

			res.addRsltAddInfo( "'ProbRepBld.loadPRData(..)' failed." );
		}
		
		return res;
	}
	
}
